/* 
 * /* 
 * EMBL - European Bioinformatics institute
 * MSD Group
 * VAMSAS Project
 *
 *Created on 14 d�c. 2005
 * @author <a href="mailto:pierre@ebi.ac.uk">Pierre MARGUERITE</a>
 * @version  1  
 * @since 14 d�c. 2005
 */
package uk.ac.ebi.msd.vamsas.vamav.session;

import uk.ac.ebi.msd.vamsas.vamav.VAMAVClient;
import uk.ac.ebi.msd.vamsas.vamav.VamaVModule;
import uk.ac.ebi.msd.vamsas.vamav.exceptions.IncorrectVamavModuleInitialisation;
import uk.ac.ebi.msd.vamsas.vamav.session.exceptions.UnableToOpenSession;
import uk.ac.ebi.msd.vamsas.vamav.util.file.FileCopy;
import uk.ac.ebi.msd.vamsas.vamav.util.file.description.FileFormatDescription;
import uk.ac.ebi.msd.vamsas.vamav.util.file.description.SimpleFileFormatDescription;
import uk.ac.ebi.msd.vamsas.vamav.util.logger.VAMAVLogger;
import uk.ac.ebi.msd.vamsas.vamav.util.logger.VAMAVLoggerImpl;
import uk.ac.ebi.msd.vamsas.vamav.util.properties.PropertyManager;
import uk.ac.ebi.msd.vamsas.vamav.util.properties.exceptions.UnableToStorePropertiesException;

import org.vamsas.client.ClientHandle;
import org.vamsas.client.IClient;
import org.vamsas.client.IClientFactory;
import org.vamsas.client.InvalidSessionUrnException;
import org.vamsas.client.NoDefaultSessionException;
import org.vamsas.client.UserHandle;
import org.vamsas.client.simpleclient.ArchiveUrn;
import org.vamsas.client.simpleclient.SessionUrn;
import org.vamsas.client.simpleclient.VamsasArchiveReader;
import org.vamsas.client.simpleclient.VamsasSession;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Array;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

/**
 * Manager for VAMSAS session.
 * Creates Session to which is attached clients
 * 
 * Avoid to reinit a session previously treated.
 * 
 * @author <a href="mailto:pierre@ebi.ac.uk">Pierre MARGUERITE</a>
 * @version 1
 * @since 14 d�c. 2005 
 * 
 */
public class SessionManager implements VamaVModule, IClientFactory
	{

/**
 * Definition string for the VAMAV client
 */
		private final String clientName = "VAMSAS - AstexViewer@MSD-EBI";
		private final String clientVersion = "0.6";
		
		/**
		 * List of  session ID and the associated session
		 *
		 */
		private Hashtable<String, VAMAVSession> sessions = null;
		
		/**
		 * Property names
		 */
		private final String modulePrefix = "sessionmanager";
		
		private final String sessionarenaProperty = "sessionarena";
		
		private final String sessiondirectoryProperty = "sessiondirectory";
		private final String sessionLogFileProperty = "sessionlogfile";
		private final String appletFileProperty = "appletfile";
		private final String sessionDocumentFileNameProperty = "sessiondocumentfilename";
		
		/*private VAMAVClientFactory clientfactory = null;*/
		 
		/**
		 * Logger to report messages
		 */
		 private final VAMAVLogger logger = new VAMAVLoggerImpl(this.getClass());
		 
		 /**
		  * Tags to replace at run-time from rules
		  */
		 private final String temporaryTag = "@temp@";
		 private final String sessionIDTag = "@sessionid@";
		 private final String sessionDirTag = "@sessiondir@";
		 private final String logExtensionTag = "@logfileextension@";
		 private final String documentExtensionTag = "@documentextension@";
		 
		 /**
		  * Rules for runtime variables (directories, files, ..)
		  */
		 
		 private String sessionArenaRule = this.temporaryTag;
		 private String sessionDirectoryRule = this.sessionIDTag;
		 
		 private String sessionLogFileRule = this.sessionDirTag+File.separator+this.sessionIDTag+"."+this.logExtensionTag;
		 private String sessionDocumentFileNameRule = this.sessionDirTag+File.separator+"vamsas"+"."+this.documentExtensionTag;
		 private String appletFile = "index.html";
		
		 /**
		  * Keep a track of opened session.
		  * Stores session
		  */
		 private Properties sessionTrack = null;
		 private final String trackFileName = "sessions.properties";
		 private final String trackDefaultDirectory = "sessions";
		 
		 /**
			 * root directory of the application
			 * Can be different of the current directory, in servlet server for example.
			 */
			private static String baseDirectory = null;


		private File sessionArena = null;
  
		 /**
		  * default constructor - called by CreateClientFactory only.
		  *
		  */
		 public SessionManager() {
		   sessionArena = null;
		 }
		 /**
		  * Create a SessionManager that works parent directory for all session directories
		  * @param path
		  */
		 public SessionManager(String path) throws IOException
		  {
		   // Check path is valid and read/writeable.
		   	File newarena = new File(path);
		   	if (newarena.exists() && newarena.isDirectory() && newarena.canRead() && newarena.canWrite()) 
		   		{
					 sessionArena = newarena;
		   		} 
		   	else
		   		{
			 		sessionArena = null;
			 		throw(new IOException("Cannot read and write to a directory called "+path));
		   		}
		 }

		/**
		 * Retrieves the  modulePrefix attribut to init the current VAMAVModule
		 *
		 * @return Returns the modulePrefix.
		 */
		public String getModulePrefix()
			{
				return this.modulePrefix;
			}



		/**
		 * Inits the Session Manager module following given properties
		 * 
		 * @see uk.ac.ebi.msd.vamsas.vamav.VamaVModule#init(java.util.Properties)
		 */
		public void init(Properties properties) throws IncorrectVamavModuleInitialisation
			{
				logger.appliInfo("Init Session Manager: ");
		/*/		try 
	    			{
	    				this.clientfactory = new VAMAVClientFactory("factory");
	    			}
	    		catch (IOException e) 
	    			{
	    				throw new  IncorrectVamavModuleInitialisation("Unable to create client factory");
	    			}*/
	    	 if (properties != null && properties.size()>0)
	    		 {
	    			 this.sessionDirectoryRule =  properties.getProperty(this.sessiondirectoryProperty, this.sessionDirectoryRule);
	    			
	    			 this.sessionLogFileRule =  properties.getProperty(this.sessionLogFileProperty, this.sessionLogFileRule);
	    			 this.appletFile =  properties.getProperty(this.appletFileProperty, this.appletFile);
	    			 this.sessionDocumentFileNameRule = properties.getProperty(this.sessionDocumentFileNameProperty, this.sessionDocumentFileNameRule);
					 this.sessionArenaRule = properties.getProperty(this.sessionarenaProperty, this.sessionArenaRule);
	    		 }
	    	 if( this.sessionArenaRule.contains(this.temporaryTag))
 				{
 					 logger.applidebug("temporary tag found");
 					try
 						{
 							File tempDir = createTempDir("VAMAV");
 							if( !logger.isProduction()) logger.applidebug("temporary directory "+tempDir.getAbsolutePath());
 							String preparePath = tempDir.getAbsolutePath().replaceAll("\\\\", "\\\\\\\\");
 							this.sessionArenaRule = this.sessionArenaRule.replaceAll(this.temporaryTag,preparePath);
 							logger.applidebug("session arena rule "+this.sessionArenaRule);
 						}
 					catch (IOException exception)
 						{
 							this.logger.appliError("Session Manager/init: Unable to create temp directory.", exception);
 							throw new IncorrectVamavModuleInitialisation(exception);
 						}
 				}
	    	 else
	    		 { logger.applidebug("No temporary tag");
	    		 	if (baseDirectory!= null)
	    		 	this.sessionArenaRule = baseDirectory.concat(File.separator).concat(this.sessionArenaRule);
	    		 }
	    	 if(this.sessionLogFileRule!=null && this.sessionLogFileRule.length()>0 )
	    		 {
	    			this.sessionLogFileRule = this.sessionLogFileRule.replaceAll(this.logExtensionTag,logger.getLogExtension());
	    		 }
	    	 
	    	 Hashtable<String, String> variables = new Hashtable<String, String>();
	    	 variables.put(this.logExtensionTag, logger.getLogExtension());
	    	 variables.put(this.sessionDirTag, this.sessionDirectoryRule);
	    	 variables.put(this.sessionDocumentFileNameRule, this.sessionDirectoryRule);
	    	 this.replaceVariable(this.sessionDocumentFileNameRule, variables);
	    	 
	    	 	this.sessionTrack = new Properties();
	    	 	PropertyManager propManager = new PropertyManager();
	    	 	this.sessionTrack = propManager.loadProperties(this.trackFileName, this.trackDefaultDirectory);
			}
		
		/**
		 * Variables which can be found in property values, by correct value 
		 * (except sessionId, which is no global to the application.) 
		 * @param rule
		 * @return
		 */
		private String replaceVariable(String rule, Hashtable<String, String> variables)
			{
				if(rule == null  )
					{
						logger.appliError("No rule provided to replace variables");
						return null;
					}
				if( rule.length()<1)
					{
						return rule;
					}
				if( variables == null || variables.size()<1)
					{
						return rule;
					}
				/**
				 * Prepare rule, otherwise '\' character is removed by replaceAll
				 */
				String preparePath = rule.replaceAll("\\\\", "\\\\\\\\");
				for (Enumeration keys = variables.keys(); keys.hasMoreElements();)
					{
						String var = (String)keys.nextElement();
						rule = rule.replaceAll(var,variables.get(var).replaceAll("\\\\", "\\\\\\\\"));
					}
				return rule;
				
			}
		
		
		/**
		 * Reopens a session following the session ID for the new user
		 * 
		 * @param firstName firstname of the user to open the session
		 * @param lastName lastname of the user
		 * @param sessionID ID of the session to open
		 * @return the created IClient according to the parameters provided
		 * @throws UnableToOpenSession if an error occurs while trying to open a session.
		 */
		public VAMAVClient openSession(String firstName, String lastName, String sessionID) throws UnableToOpenSession
			{
				if(sessionID == null || sessionID.length()<1 )
					{
						this.logger.sessionFatal("Open Session - No sessionID provided: can not open session.");
						return null;
					}
				VAMAVSession session = this.getSessions().get(sessionID);
				VAMAVClient vorbaclient = null;
				if (session == null)
					{
						//this.logger.sessionFatal("Open Session - No previous session exists. Can not open session.");
						//return null;
						this.logger.appliInfo("Opening closed session");
						String sessionDir = this.sessionTrack.getProperty(sessionID);
						if( sessionDir == null || sessionDir.length()<1 )
							{
								this.logger.appliFatal("Open Session - No previous session exists. Can not open session.");
								return null;
							}
						File dir = new File (sessionDir);
						String appletFileName = this.appletFile.replaceAll(this.sessionIDTag, sessionID);
						if (dir.exists() && dir.canRead() && dir.isDirectory() && Arrays.asList(dir.list()).contains(appletFileName))
							{//the directory contains some files
//								creating VAMSAS CLient
								try
									{
										session = new VAMAVSession(dir);
										//session.setSessionDirectory(new File (sessionDir));
										String preparePath = sessionDir.replaceAll("\\\\", "\\\\\\\\");
										session.setSessionLogFile(new File (this.sessionLogFileRule.replaceAll(this.sessionIDTag, sessionID).replaceAll(this.sessionDirTag, preparePath)));
							    		session.setAppletFileName(appletFileName);
							    		session.setClosedSession(true);//.setSessionID( true);
						        		this.getSessions().put(sessionID, session);
									}
									catch (IOException ioe)
									{
										this.logger.appliError("unable to access to the session directory.", ioe);
										throw new UnableToOpenSession (ioe);
									}
							}
					}
				ClientHandle app = new ClientHandle(this.clientName,this.clientVersion);
						
				if( firstName != null)
					{
						UserHandle user = new UserHandle(firstName,lastName);
						vorbaclient = (VAMAVClient)this.getIClient(app, user, sessionID);
					}
				else
					{
						/*try
							{*/
								vorbaclient = (VAMAVClient)this.getIClient(app);
								/*		}
					catch (NoDefaultSessionException ndse)
							{
								this.logger.sessionError("Can not create client: ",ndse);
								this.logger.appliError("Can not create client: ",ndse);
								throw new UnableToOpenSession(ndse);
							}*/
					}
				session.addAttachedClient(vorbaclient);
				return vorbaclient;
			}
		
		/**
		 * Opens a new VAMSAS session for the provided VAMSAS document
		 * 
		 * 
		 * @param firstName firstname of the user to open the session
		 * @param lastName lastname of the user
		 * @param sessionID ID of the session to open
		 * @return the created IClient according to the parameters provided
		 * @throws UnableToOpenSession if an error occurs while trying to open a session.
		 */
		public VAMAVClient openSession(String firstName, String lastName, File documentPath) throws UnableToOpenSession
			{
				/**
				 * check document path
				 */
				if(documentPath == null || !documentPath.exists()) 
					{
						this.logger.sessionFatal("No Document path provided or path incorrect. Can not open session.");
						return null;
					}
		
				if (!documentPath.canRead() || !documentPath.isFile()) 
					{
						this.logger.sessionFatal("No Document path provided or path incorrect. Can not open session. "+documentPath);
						return null;
					}
					
					if ( (firstName == null ||firstName.length()<1 )
						&& lastName == null||lastName.length()<1)
					{
						logger.appliError("No user information provided. Can not open session");
						throw new UnableToOpenSession("No user information provided. Can not open session");
					}
				logger.applidebug("Starting new session with data from "+documentPath+".");
				//ArchiveUrn vamdoc = new ArchiveUrn(documentPath);
				VamsasArchiveReader archive = new VamsasArchiveReader(documentPath);
				   // TODO: a real validity test. The below just checks it can be read.
				if (!archive.isValid())
					 throw new UnableToOpenSession(documentPath+" is not a valid vamsasDocument archive.");
				ArchiveUrn vamsasdocument = null;
				try
					{
							vamsasdocument = new ArchiveUrn(documentPath);
					}
				catch (MalformedURLException mue)
					{
						throw new UnableToOpenSession();
					}

			
				ClientHandle app = new ClientHandle(this.clientName,this.clientVersion);
				UserHandle user = new UserHandle(firstName,lastName);

				try
					{
						this.logger.applidebug("Opening session for "+documentPath + " "+vamsasdocument.asFile() +" "+vamsasdocument.getSessionUrn());
						return (VAMAVClient) this.openSession(app, user, vamsasdocument);
					}
				catch (IOException ioe)
				{
					throw new UnableToOpenSession(ioe);
				}
			}
			
			
			
		public IClient openSession(ClientHandle applicationHandle, UserHandle userId, ArchiveUrn vamsasdocument) throws IOException
		 {
			 this.logger.applidebug("openSession "+vamsasdocument.asFile());
 				if (vamsasdocument == null) 
				{
					return null;
				}
			
				// create new session directory
				if (sessionArena==null)
					throw new Error("Improperly initialised SimpleClientFactory object - null sessionArena.");
				File sessdir = File.createTempFile("sess", ".simpleclient", sessionArena);
				if (!(sessdir.delete() && sessdir.mkdir()))
				   throw new IOException("Could not make session directory "+sessdir);
								
//				create session
							//generate document copy in the session directory
				VAMAVSession session = null;
				try
				  {this.logger.applidebug("Creating VAMAVSession for "+sessdir);
					  session = new VAMAVSession(sessdir);
					  this.sessionTrack.setProperty(session.getSessionID(), sessdir.getAbsolutePath());
					String preparePath = sessdir.getAbsolutePath().replaceAll("\\\\", "\\\\\\\\");
	        	
					  session.setSessionLogFile(new File (this.sessionLogFileRule.replaceAll(this.sessionIDTag, session.getSessionID()).replaceAll(this.sessionDirTag, preparePath)));
					  session.setAppletFileName(this.appletFile);
					  this.getSessions().put(session.getSessionID(), session);
					// copy document into session directory
					  File doc = vamsasdocument.asFile();
					  this.logger.applidebug("setting document "+doc.getPath());
					  this.logger.applidebug("setting document "+doc.getAbsolutePath());
		        		session.setVamsasDocument(doc);
				  }
				catch (IOException ioe)
				  {
				    throw ioe;//new UnableToOpenSession(ioe);	
				  }
				IClient vorbaclient = this.getIClient(applicationHandle, userId, session);
				session.addAttachedClient((VAMAVClient)vorbaclient);
			//process the Vamsas Document to launch AstexViewer with suitable parameters
	        	return vorbaclient;
			}
		
		/**
		   * make a new vamsas session from the data in the archive vamsasdocument 
		   * @param applicationHandle
		   * @param userId
		   * @param vamsasdocument
		   * @return
		   */
		/*  public IClient openSession(ClientHandle applicationHandle, UserHandle userId, ArchiveUrn vamsasdocument) throws IOException {
			// verify applicationHandle and userId
			// TODO: verify applicationHandle and userId
			// verify vamsasdocument is a valid document.
			File vamdoc = vamsasdocument.asFile();
			this.logger.applidebug("Starting new session with data from "+vamsasdocument.getSessionUrn()+"(File is : "+vamdoc+")");
			VamsasArchiveReader archive = new VamsasArchiveReader(vamdoc);
			// TODO: a real validity test. The below just checks it can be read.
			if (!archive.isValid())
			  throw new IOException(vamsasdocument.getSessionUrn()+" is not a valid vamsasDocument archive.");
			// create new session directory
			if (sessionArena==null)
			  throw new Error("Improperly initialised SimpleClientFactory object - null sessionArena.");
			File sessdir = File.createTempFile("sess", ".simpleclient", sessionArena);
			if (!(sessdir.delete() && sessdir.mkdir()))
				throw new IOException("Could not make session directory "+sessdir);
			VAMAVSession sess = new VAMAVSession(sessdir);
			// copy document into session directory
			sess.setVamsasDocument(vamsasdocument.asFile());
			// create client instance and return.
			VAMAVClient client = new VAMAVClient(userId, applicationHandle, sess);
    
			return client;
		  }*/
		
		/**
		 * opens session according to a provided sessionID or reads a provided VAMSAS document 
		 * 
		 * if no sessionID provided, creates a new session and opens the provided document.
		 * if the session Id is unknown, creates a new session and opens the provided document.
		 * if the session exists, and the provided document is newer, the provide document will replaced 
		 * the current document in use.
		 * if the session has ben closed, the document is opened in a new session.
		 * @param firstName user   firstname
		 * @param lastName user lastname
		 * @param documentPath path to a VAMSAS document
		 * @param sessionID ID of the session to open
		 * @return a VAMAVClient correspoding to the given parameters
		 * @throws UnableToOpenSession raises if an error during access to the document file
		 */
		public VAMAVClient openSession(String firstName, String lastName, File documentPath, String sessionID) throws UnableToOpenSession
			{
				if(sessionID == null || sessionID.length()<1 )
					{
						logger.sessionWarn("Open Session - No sessionID provided: can not open session.");
						return this.openSession(firstName, lastName, documentPath);
					}
				VAMAVSession session = this.getSessions().get(sessionID);
				if (session == null)
					{
						return this.openSession(firstName, lastName, documentPath);
					}
				
			/*
								VAMAVSession session = null;
								try
								{
									session = new VAMAVSession(new File(sessionDir));
								}
								catch (IOException ioe)
								{
									throw new UnableToOpenSession(ioe);	
								}*/
	        
						
				VAMAVClient newClient = null;
				if (session.isOpen() && !session.isClosed())
					{
						FileFormatDescription description = new SimpleFileFormatDescription(documentPath);
				        Hashtable<String, String> variables = new Hashtable<String, String>();
				        variables.put(this.documentExtensionTag, description.getFileExtension());
				       // String sessionDocumentPath = this.replaceVariable(this.sessionDocumentFileNameRule, variables);
					   File sessionDoc = session.getVamsasDocumentFile();
						//File sessionDoc = new File (sessionDocumentPath);
						if (documentPath != null && documentPath.canRead() && documentPath.isFile() && documentPath.lastModified()< sessionDoc.lastModified())
							{//the provided document is newer than the document in the session directory.
							logger.sessionInfo("The provided document is newer than the previously treated document. Proceed new document.");
									return this.openSession(firstName, lastName, documentPath);
								}
							newClient = (VAMAVClient)session.getAttachedClient(0).clone();
							if( firstName != null && firstName.length()<1 && lastName!=null && lastName.length()<1)
				        		{
				        			UserHandle user = new UserHandle(firstName,lastName);
				        			newClient.setUser(user);
				        		}
							try
								{
									FileCopy copyManager = new FileCopy(session.getSessionID());
									File newDoc = sessionDoc;
									copyManager.copy(documentPath, newDoc);
									documentPath = newDoc;
									copyManager = null;
									session.setVamsasDocumentFile(newDoc);
								}
							catch (IOException ioe)
								{
									logger.appliError("Unable to copy VAMSAS Document.", ioe);
								}	
					}
				else
					newClient = this.openSession(firstName, lastName, documentPath);
				/*String sessionID = vorbaclient.getSessionUrn();
						//					String sessionDir = this.sessionDirectoryRule.replaceAll(this.sessionIDTag, sessionID);
						//					String preparePath = sessionDir.replaceAll("\\\\", "\\\\\\\\");
						//					logger.applidebug("session directory "+sessionDir);
											String appletFileName = this.appletFile.replaceAll(this.sessionIDTag, sessionID);
				
			//	this.sessionTrack.setProperty(sessionID, sessionDir);
						//			session.setSessionDirectory(new File (sessionDir));
			//						session.setSessionLogFile(new File (this.sessionLogFileRule.replaceAll(this.sessionIDTag, sessionID).replaceAll(this.sessionDirTag, preparePath)));
							//		session.setSessionID(sessionID,false);
									session.addAttachedClient(vorbaclient);
									session.setAppletFileName(appletFileName);
									this.getSessions().put(sessionID, session);
	        	
									//Copying vamsas document file to session directory
									FileFormatDescription description = new SimpleFileFormatDescription(documentPath);
									Hashtable<String, String> variables = new Hashtable<String, String>();
									variables.put(this.documentExtensionTag, description.getFileExtension());
									variables.put(this.sessionDirTag, session.getSessionDirectory().getAbsolutePath());
									String sessionDocumentPath = this.replaceVariable(this.sessionDocumentFileNameRule, variables);
									logger.sessionDebug("Session document "+ sessionDocumentPath);
									//String sessionDocumentPath = this.sessionDocumentFileNameRule.replaceAll(preparePath, appletFileName);//sessionDir+File.separator+"vamsas"+"."+description.getFileExtension();
									try
										{
											FileCopy copyManager = new FileCopy(sessionID);
											File newDoc = new File(sessionDocumentPath);
											copyManager.copy(documentPath, newDoc);
											documentPath = newDoc;
											copyManager = null;
											session.setVamsasDocumentFile(newDoc);
										}
									catch (IOException ioe)
										{
											logger.appliError("Unable to copy VAMSAS Document.", ioe);
										}	
				*/
				session.addAttachedClient(newClient);
				return newClient;
			}
	    
	    
	    /**
	     * Creates a new session of a given user.
	     * 
	     * @param firstName first name of the user to open the session for
	     * @param lastName lastname of the user 
	     * @return the created IClient according to the parameters provided
	     * @throws UnableToOpenSession raises if an error occurs during the accessing to the VAMSAS document.
	     *///DONE 14/03/2006
		public VAMAVClient createSession(String firstName, String lastName) throws UnableToOpenSession
			{
				// create new session directory
				if (sessionArena==null)
					throw new Error("Improperly initialised SimpleClientFactory object - null sessionArena.");
				File sessdir =  null;
				try
					{
						sessdir = File.createTempFile("sess", ".simpleclient", sessionArena);
						if (!(sessdir.delete() && sessdir.mkdir()))
				   		throw new UnableToOpenSession("Could not make session directory "+sessdir);
					}
				catch (IOException ioe)
				{
					throw new UnableToOpenSession("Could not make session directory.");
				
				}
								
//				create session
				//generate document copy in the session directory
				VAMAVSession session = null;
				//VAMAVClient vorbaclient = null;
				try
				   {
					  session = new VAMAVSession(sessdir);
					  this.sessionTrack.setProperty(session.getSessionID(), sessdir.getAbsolutePath());
					String preparePath = sessdir.getAbsolutePath().replaceAll("\\\\", "\\\\\\\\");
	        	
					  session.setSessionLogFile(new File (this.sessionLogFileRule.replaceAll(this.sessionIDTag, session.getSessionID()).replaceAll(this.sessionDirTag, preparePath)));
					  session.setAppletFileName(this.appletFile);
					  this.getSessions().put(session.getSessionID(), session);
				  }
				catch (IOException ioe)
				  {
					throw new UnableToOpenSession(ioe);	
				  }
				ClientHandle app = new ClientHandle(this.clientName,this.clientVersion);
				UserHandle user = new UserHandle(firstName,lastName);
				IClient vorbaclient = this.getIClient(app, user, session);
				session.addAttachedClient((VAMAVClient)vorbaclient);
				//process the Vamsas Document to launch AstexViewer with suitable parameters
				return (VAMAVClient) vorbaclient;

			}
		
		
		
			/**
			  * Create a new VAMAVCLient for a given clientHandle
			  * 
			  * @see org.vamsas.client.IClientFactory#getIClient(org.vamsas.client.ClientHandle)
			  */
			 public IClient getIClient(ClientHandle applicationHandle) //throws NoDefaultSessionException
				 {	
					 try
						 {
							 return new VAMAVClient(applicationHandle);
						 }
					 catch (InvalidSessionUrnException isue)
						 {
							 this.logger.appliError("Unable to get client ",isue);
							 this.logger.sessionError("Unable to get client ",isue);
							// throw new NoDefaultSessionException(isue);
						 }
					 return null;
				 }

			 /**
			  * 
			  * Retrieves a client of a given session URN.
			  * If does not exist, create a new one.
			  * 
			  * @param applicationHandle client handle object
			  * @param sessionUrn URN of the session to attach the client
			  * @see org.vamsas.client.IClientFactory#getIClient(org.vamsas.client.ClientHandle, java.lang.String)
			  */
			 //done 14 03 06
			 public IClient getIClient(ClientHandle applicationHandle, String sessionUrn)
				 {
					 // applicationHandle also contains sessionURN
			   // 	applicationHandle.getClientUrn()
    
					/*VAMAVClient client = null; /* this.getClient(sessionUrn);
					 if (client == null)
						 {
							 client = new VAMAVClient(applicationHandle);
						 }*/
					 VAMAVClient client = null;
					 try
						 {
							 VAMAVSession session =  this.getSessions().get(sessionUrn);
							 client = new VAMAVClient(applicationHandle);
							 session.addAttachedClient(client);
							 client.setVAMAVSession(session);
						 }
					 catch (InvalidSessionUrnException isUExc)
						 {
							 this.logger.sessionError("The provided Session URN is incorrect.", isUExc);
							 this.logger.appliError("The provided Session URN is incorrect.", isUExc);
							 client = null;
						 }
					 return client;
			 }

			 /**
			  * DONE 14/03/2006
			  * @see org.vamsas.client.IClientFactory#getIClient(org.vamsas.client.ClientHandle, org.vamsas.client.UserHandle, java.lang.String)
			  */
			 public IClient getIClient(ClientHandle applicationHandle, UserHandle userId,
				 String sessionUrn) 
				 {
					/* VAMAVClient client = this.getClient(sessionUrn);
					 if (client == null)
						 {
							 client = new VAMAVClient(applicationHandle);
							 client.setUser(userId);
						 }
					 if (userId == null)
						 {
							 logger.appliError("There is non userID provided.");
						 }
					 else
						 {
							 /*this.addUser(sessionUrn,userId);*/
						 /*}*/
					 VAMAVClient client = null;
					 try
						 {
							 VAMAVSession session =  this.getSessions().get(sessionUrn);
							 client = new VAMAVClient(userId, applicationHandle, session);
							 session.addAttachedClient(client);
							 client.setVAMAVSession(session);
						 }
					 catch (InvalidSessionUrnException isUExc)
						 {
							 this.logger.sessionError("The provided Session URN is incorrect.", isUExc);
							 this.logger.appliError("The provided Session URN is incorrect.", isUExc);
							 client = null;
						 }
					 return client;
				 }
 
			 /**
			  * DONE
			  * @see org.vamsas.client.IClientFactory#getIClient(org.vamsas.client.ClientHandle, org.vamsas.client.UserHandle)
			  */
			 public VAMAVClient getIClient(ClientHandle applicationHandle, UserHandle userId, VAMAVSession session) 
				 {
					 if (userId == null)
						 {
							 logger.appliInfo("There is non userID provided.");
						 }
					 else
						 {
						/*	 String clientUrn = applicationHandle.getClientUrn();
							 if (clientUrn != null && clientUrn.length()>0)
								 this.addUser(clientUrn, userId);*/
						 }
					if (session == null)
					   {
						   this.logger.appliError("No session provided. Can not create client if not associated to a session.");
						   return null;
					   }
					
					VAMAVClient client = null;
					try
						{
							// String sessionID = new SessionHandle().generateSessionID();
							client = new VAMAVClient(userId, applicationHandle, session);
							client.setUser(userId);
						}
					 catch (InvalidSessionUrnException isUExc)
						 {
							 this.logger.sessionError("Unable to create session due to incorrect session.", isUExc);
							 this.logger.appliError("Unable to create session due to incorrect session.", isUExc);
							 
							 client = null;
						 }
					// this.addClient(client.getSessionUrn(), client );
					// applicationHandle.setClientUrn(sessionID);
					 logger.appliInfo("generated sessionID "+client.getSessionUrn());
					 return client;
				 }
		
		/**
		 * Remove a client
		 * 
		 * Do nothing WHY?
		 *  at a later stage?
		 * @param client the client to remove
		 */
		public void RemoveClient(VAMAVClient client)
			{
				if (client == null)
					{
						this.logger.appliError("SessionManager - Can not remove client. Client is null");
						return;
					}
				
			}
		
		/**
		 * 
		 * Finalize the current manager
		*/
		@Override
		protected void finalize()throws Throwable 
		  	{
		  		try 
		  			{
		  				this.kill();
		  			} finally {
		  				super.finalize();
		  			}
		  	}
		
		/**
		 * Kill the current manager
		 *
		 */
		private void kill()
			{
				this.close();
			}


/**
 * Closes the current manager.
 * registers required data
 *
 */
public void close()
	{
		if( this.sessionTrack != null && !this.sessionTrack.isEmpty())
			{
				PropertyManager propManager = new PropertyManager();
				try
					{
						propManager.storeProperties(trackFileName, sessionTrack, "", trackDefaultDirectory);
					}
				catch (UnableToStorePropertiesException utspe)
					{
						logger.appliWarn("Unable to store session track");
					}
				 this.sessionTrack.clear();
			}
		 this.sessionTrack = null;
		if (this.sessions!= null)
			{
				this.sessions.clear();
				this.sessions = null;
			}
	}
		
		/**
		 * Retrieves the  sessions attribut
		 *
		 * @return Returns the sessions.
		 */
		private Hashtable<String, VAMAVSession> getSessions()
			{
				if( this.sessions == null)
					this.sessions = new Hashtable<String, VAMAVSession>();
				return this.sessions;
			}
		
		/**
		 * Creates a temp directory in a given directory.
		 * 
		 * @param prefix prefix of the directory to create
		 * @param directory
		 * @throws IOException if an error occurs during file creation
		 * @return the temporary directory
		 */    
		public  File createTempDir(String prefix, File directory) throws IOException
		  {
			  if( prefix == null)
				  {
					  logger.sessionError("Error: no prefix provided to create temp file.");
					  throw new IOException();
				  }
		      File tempFile = null;
		    	  	if( directory == null )
		    	  		tempFile = File.createTempFile(prefix, "");
		    	  	else
		    	  		tempFile = File.createTempFile(prefix, "", directory);
		      if (!tempFile.delete())
		          throw new IOException();
		      if (!tempFile.mkdir())
		          throw new IOException();
		      return tempFile;        
		  }
		
		  /**
		   * Creates a temp directory in the System temp directory.
		   * @param prefix prefix of the temp directory name
		   * @return the create file
		   * @throws IOException if an error during file creation
		   * 
		   */    
		public  File createTempDir(String prefix) throws IOException
		{
			      return this.createTempDir(prefix, null);
		}
		  
		/**
		 * Close the current session manager
		 *
		 */
		public void closeManager()
			{
				if (this.sessions != null)
					{
						for ( Enumeration ids = this.sessions.keys() ; ids.hasMoreElements();)
							{
								String sessionID = (String)ids.nextElement();
								if( sessionID == null || sessionID.length()<1)continue;
								VAMAVSession session = this.sessions.remove(sessionID);
								if (session != null)
									{
										session.close();
										session = null;
									}
							}
						this.sessions.clear();
						this.sessions = null;
					}
				
			}
		 /**Sets baseDirectory attribut
		 *
		 * @param _baseDirectory The baseDirectory to set.
		 */
		public static void setBaseDirectory(String _baseDirectory)
			{
				baseDirectory = _baseDirectory;
				if( baseDirectory != null && baseDirectory.length()>0 && !baseDirectory.endsWith(File.separator))
					{
						baseDirectory = baseDirectory.concat(File.separator);
					}
			}
		
		/**
		 * Retrieves the list of available sessions for the current session manager
		 */
		public ArrayList<String> getAvailableSessions()
		{
			ArrayList<String> sessionList  = null;
			for (Enumeration<String>  sessionIDs = sessions.keys();sessionIDs.hasMoreElements();)
				{
					String sessionID = sessionIDs.nextElement();
					if (sessionID == null || sessionID.length()<1)
						{
							continue;
						}
					if (sessionList == null) sessionList = new ArrayList<String>();
					sessionList.add(sessionID);
				}
			return sessionList;
		}
	/* (non-Javadoc)
	 * @see org.vamsas.client.IClientFactory#getIClient(org.vamsas.client.ClientHandle, org.vamsas.client.UserHandle)
	 */
	public IClient getIClient(
		ClientHandle applicationHandle,
		UserHandle userId) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	   * @see org.vamsas.client.IClientFactory#getCurrentSessions()
	   */
	  public String[] getCurrentSessions() {
	    // TODO look in the arena and enumerate session handles for return.
	    return new String[] {};
	  }
	}
